package test.jpa;
import javax.persistence.EntityManager;
import javax.persistence.EntityManagerFactory;
import javax.persistence.Persistence;

import test.jpa.entity.Address;
import test.jpa.entity.Base;
import test.jpa.entity.Person;

public class Task {
	interface IEntityCreator<T extends Base> {
		T loadOrCreate(final EntityManager em, final String id);
	}
	static class PartnerCreator implements IEntityCreator<Person> {
		@Override
		public Person loadOrCreate(final EntityManager em, final String id) {
			final Person person = em.find(Person.class, id); // try to find ...
			return (null != person) ? person : new Person(id); // ... or create new one
		}
	}
	static class AddressCreator implements IEntityCreator<Address> {
		@Override
		public Address loadOrCreate(final EntityManager em, final String id) {
			final Address address = em.find(Address.class, id); // try to find ...
			return (null != address) ? address : new Address(id); // ... or create new one
		}
	}

	public void write(int run) {
		final long t1 = System.currentTimeMillis();
		
		final EntityManagerFactory emf = Persistence.createEntityManagerFactory("myDbFile.odb");
		final EntityManager em = emf.createEntityManager();

		try {
			for(int idxOuter = 0; idxOuter < 50; idxOuter++) {
				em.getTransaction().begin();

				for(int idxInner = 0; idxInner < 200; idxInner++) {

					final String key = idxOuter + ":" + idxInner;
					final Person person = new PartnerCreator().loadOrCreate(em, key);
					final Address address = new AddressCreator().loadOrCreate(em, key + ":" + run);

					person.addAddress(address);

					em.persist(person);
				}

				em.getTransaction().commit();
				em.clear();
				
				System.out.print('.');
			}
		}
		catch (final Exception ex) {
			ex.printStackTrace();
		}

		em.close();
		emf.close();
		
		final long t2 = System.currentTimeMillis();
		System.out.println(t2-t1);

	}
}
